@@ -1,4 +1,5 @@
 from .byrows_update import byrows_update
+from .duplicates import find_duplicates_action
 from .export import (export_as_csv, export_as_fixture,
                      export_as_xls, export_delete_tree,)
 from .graph import graph_queryset
@@ -9,6 +10,7 @@ actions = [export_as_fixture,
            export_as_csv,
            export_as_xls,
            export_delete_tree,
+           find_duplicates_action,
            merge,
            mass_update,
            graph_queryset,
