@@ -1,29 +1,29 @@
 from django.apps import apps
-from django.conf import settings
 
-AA_PERMISSION_CREATE_USE_SIGNAL = 1
-AA_PERMISSION_CREATE_USE_APPCONFIG = 2
-AA_PERMISSION_CREATE_USE_COMMAND = 3
-AA_PERMISSION_HANDLER_VALUES = [AA_PERMISSION_CREATE_USE_SIGNAL,
-                                AA_PERMISSION_CREATE_USE_APPCONFIG,
-                                AA_PERMISSION_CREATE_USE_COMMAND]
-
-AA_PERMISSION_HANDLER = getattr(settings, 'AA_PERMISSION_HANDLER', AA_PERMISSION_CREATE_USE_SIGNAL)
+__all__ = ["create_extra_permissions"]
 
 
 def get_permission_codename(action, opts):
     return '%s_%s' % (action, opts.object_name.lower())
 
 
+def get_contenttype_for_model(model):
+    from django.contrib.contenttypes.models import ContentType
+    model = model._meta.concrete_model
+    opts = model._meta
+    ct, __ = ContentType.objects.get_or_create(
+        app_label=opts.app_label,
+        model=opts.model_name,
+    )
+    return ct
+
+
 def create_extra_permissions():
     from django.contrib.auth.models import Permission
     from django.contrib.contenttypes.models import ContentType
 
     from .actions import actions as aa
 
-    #  ('adminactions_export', 'adminactions_massupdate',
-    #                        'adminactions_merge', 'adminactions_chart',
-    #                        'adminactions_byrowsupdate')
     perm_suffix = 'adminactions_'
     existing_perms = set(
         Permission.objects.filter(codename__startswith=perm_suffix).values_list(
