@@ -1,4 +1,9 @@
 import os
+from environ import Env
+
+from adminactions.consts import AA_PERMISSION_CREATE_USE_COMMAND
+
+env = Env()
 
 here = os.path.dirname(__file__)
 
@@ -78,7 +83,7 @@ LANGUAGE_CODE = 'en-us'
 SITE_ID = 1
 USE_I18N = True
 USE_L10N = True
-USE_TZ = False
+USE_TZ = True
 MEDIA_ROOT = os.path.join(here, 'media')
 MEDIA_URL = ''
 STATIC_ROOT = os.path.join(here, 'static')
@@ -96,9 +101,6 @@ MESSAGE_STORAGE = 'demo.storage.PlainCookieStorage'
 
 ROOT_URLCONF = 'demo.urls'
 
-
-from adminactions.perms import AA_PERMISSION_CREATE_USE_COMMAND  # noqa
-
 AA_PERMISSION_HANDLER = AA_PERMISSION_CREATE_USE_COMMAND
 
 INSTALLED_APPS = [
@@ -173,3 +175,14 @@ AUTH_PASSWORD_VALIDATORS = [
     #     'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
     # },
 ]
+
+# CELERY STUFF
+BROKER_URL = env("CELERY_BROKER_URL")
+# CELERY_RESULT_BACKEND = env("CELERY_RESULT_BACKEND")
+# CELERY_ACCEPT_CONTENT = ["application/json"]
+# CELERY_TASK_SERIALIZER = "json"
+CELERY_TASK_ALWAYS_EAGER = env("CELERY_ALWAYS_EAGER")
+# CELERY_ALWAYS_EAGER = env("CELERY_ALWAYS_EAGER")
+# CELERY_RESULT_SERIALIZER = "json"
+# CELERY_TIMEZONE = TIME_ZONE
+# CELERY_BEAT_SCHEDULER = "django_celery_beat.schedulers:DatabaseScheduler"
