@@ -1,5 +1,5 @@
-from django.conf.urls import include, re_path
 from django.contrib import admin
+from django.urls import include, re_path
 
 from adminactions import actions
 
